@@ -502,8 +502,12 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 		// uncalibrated).
 		WARNING (abstract->context, "Disabled all O2 sensors due to a default calibration value.");
 		parser->calibrated = 0;
+		if (mode != DC_DIVEMODE_OC)
+			add_string(parser, "PPO2 source", "voted/averaged");
 	} else {
 		parser->calibrated = data[86];
+		if (mode != DC_DIVEMODE_OC)
+			add_string(parser, "PPO2 source", "cells");
 	}
 
 	// Cache the data for later use.
@@ -669,19 +673,19 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 		if ((status & OC) == 0) {
 			// PPO2
 			if ((status & PPO2_EXTERNAL) == 0) {
-#ifdef SENSOR_AVERAGE
-				sample.ppo2 = data[offset + 6] / 100.0;
-				if (callback) callback (DC_SAMPLE_PPO2, sample, userdata);
-#else
-				sample.ppo2 = data[offset + 12] * parser->calibration[0];
-				if (callback && (parser->calibrated & 0x01)) callback (DC_SAMPLE_PPO2, sample, userdata);
+				if (!parser->calibrated) {
+					sample.ppo2 = data[offset + 6] / 100.0;
+					if (callback) callback (DC_SAMPLE_PPO2, sample, userdata);
+				} else {
+					sample.ppo2 = data[offset + 12] * parser->calibration[0];
+					if (callback && (parser->calibrated & 0x01)) callback (DC_SAMPLE_PPO2, sample, userdata);
 
-				sample.ppo2 = data[offset + 14] * parser->calibration[1];
-				if (callback && (parser->calibrated & 0x02)) callback (DC_SAMPLE_PPO2, sample, userdata);
+					sample.ppo2 = data[offset + 14] * parser->calibration[1];
+					if (callback && (parser->calibrated & 0x02)) callback (DC_SAMPLE_PPO2, sample, userdata);
 
-				sample.ppo2 = data[offset + 15] * parser->calibration[2];
-				if (callback && (parser->calibrated & 0x04)) callback (DC_SAMPLE_PPO2, sample, userdata);
-#endif
+					sample.ppo2 = data[offset + 15] * parser->calibration[2];
+					if (callback && (parser->calibrated & 0x04)) callback (DC_SAMPLE_PPO2, sample, userdata);
+				}
 			}
 
 			// Setpoint
